package com.gic.haoban.manage.service.dao.mapper.hm;

import com.gic.haoban.manage.service.entity.hm.TabHaobanHmClerkRelation;
import com.gic.haoban.manage.service.pojo.bo.hm.HmClerkRelationBO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 多人活码(TabHaobanHmClerkRelation)表数据库访问层
 *
 * @author mozhu
 * @since 2022-07-04 15:42:20
 */
public interface TabHaobanHmClerkRelationMapper {

    /**
     * 通过ID查询单条数据
     *
     * @param relationId 主键
     * @return 实例对象
     */
    TabHaobanHmClerkRelation queryById(Long relationId);


    /**
     * 新增数据
     *
     * @param tabHaobanHmClerkRelation 实例对象
     * @return 影响行数
     */
    int insert(TabHaobanHmClerkRelation tabHaobanHmClerkRelation);

    /**
     * 批量新增数据（MyBatis原生foreach方法）
     *
     * @param entities List<TabHaobanHmClerkRelation> 实例对象列表
     * @return 影响行数
     */
    int insertBatch(@Param("entities") List<TabHaobanHmClerkRelation> entities);


    /**
     * 修改数据
     *
     * @param tabHaobanHmClerkRelation 实例对象
     * @return 影响行数
     */
    int update(TabHaobanHmClerkRelation tabHaobanHmClerkRelation);


    /**
     * 更新状态通过hm id
     *
     * @param hmId   hm id
     * @param status 状态
     * @return int
     * @author mozhu
     * @date 2022-07-05 16:26:09
     */
    int updateStatusByHmId(@Param("hmId") Long hmId, @Param("status") Integer status);

    /**
     * 查询hm id和职员id
     *
     * @param hmId
     * @param clerkId
     * @return {@link HmClerkRelationBO }
     * @author mozhu
     * @date 2022-07-07 13:33:22
     */
    TabHaobanHmClerkRelation queryByHmIdAndClerkId(@Param("hmId") Long hmId, @Param("clerkId") String clerkId);

    /**
     * 更新 addnum
     *
     * @param relationId 关系id
     * @param overFlag   是否上限1是0否
     * @return int
     * @author mozhu
     * @date 2022-07-07 13:40:00
     */
    int updateAddNumById(@Param("relationId") Long relationId, @Param("overFlag") Integer overFlag);

    /**
     * 重置
     *
     * @author mozhu
     * @date 2022-07-07 13:53:36
     */
    void resetAddNumAndOverFlag();

    /**
     * 查询职员id
     *
     * @param clerkId      职员id
     * @param enterpriseId 企业标识
     * @return {@link List }<{@link TabHaobanHmClerkRelation }>
     * @author mozhu
     * @date 2022-07-08 14:49:18
     */
    List<TabHaobanHmClerkRelation> queryByClerkId(@Param("clerkId") String clerkId,@Param("enterpriseId") String enterpriseId);

    /**
     * 根据导购id获取hmid
     *
     * @param hmId hm id
     * @return {@link List }<{@link String }>
     * @author mozhu
     * @date 2022-07-08 14:56:59
     */
    List<String> getNumByHmId(@Param("hmId") Long hmId);


    /**
     * 根据导购信息获取hmid
     *
     * @param hmId hm id
     * @return {@link List }<{@link TabHaobanHmClerkRelation }>
     * @author mozhu
     * @date 2022-07-13 17:04:38
     */
    List<TabHaobanHmClerkRelation> getHmClerkByHmId(@Param("hmId") Long hmId);

}

