package com.gic.haoban.manage.service.dao.mapper.hm;

import com.gic.haoban.manage.api.dto.qdto.hm.HmQrcodeListQDTO;
import com.gic.haoban.manage.service.entity.hm.TabHaobanHmQrcode;
import com.gic.haoban.manage.service.pojo.bo.hm.HmQrcodeBO;
import com.gic.haoban.manage.service.pojo.bo.hm.HmQrcodeGroupNumBO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 活码(TabHaobanHmQrcode)表数据库访问层
 *
 * @author mozhu
 * @since 2022-07-04 15:42:20
 */
public interface TabHaobanHmQrcodeMapper {

    /**
     * 通过ID查询单条数据
     *
     * @param hmId 主键
     * @return 实例对象
     */
    TabHaobanHmQrcode queryById(@Param("hmId") Long hmId);

    /**
     * 分页列表
     *
     * @param hmQrcodeListQDTO 查询条件
     * @return 对象列表
     */
    List<TabHaobanHmQrcode> queryListByPage(HmQrcodeListQDTO hmQrcodeListQDTO);

    /**
     * 新增数据
     *
     * @param tabHaobanHmQrcode 实例对象
     * @return 影响行数
     */
    int insert(TabHaobanHmQrcode tabHaobanHmQrcode);

    /**
     * 批量新增数据（MyBatis原生foreach方法）
     *
     * @param entities List<TabHaobanHmQrcode> 实例对象列表
     * @return 影响行数
     */
    int insertBatch(@Param("entities") List<TabHaobanHmQrcode> entities);

    /**
     * 修改数据
     *
     * @param tabHaobanHmQrcode 实例对象
     * @return 影响行数
     */
    int update(TabHaobanHmQrcode tabHaobanHmQrcode);

    /**
     * 通过主键删除数据
     *
     * @param hmId 主键
     * @return 影响行数
     */
    int deleteById(Long hmId);


    /**
     * 更新状态通过id
     *
     * @param statusFlag 状态标志
     * @param hmId       hm id
     * @return int
     * @author mozhu
     * @date 2022-07-06 13:42:07
     */
    int updateStatusById(@Param("statusFlag") Integer statusFlag, @Param("hmId") Long hmId);

    /**
     * 通过职员id列表
     *
     * @param clerkId      职员id
     * @param enterpriseId 企业标识
     * @return {@link List }<{@link HmQrcodeBO }>
     * @author mozhu
     * @date 2022-07-06 15:36:20
     */
    List<TabHaobanHmQrcode> listByClerkId(@Param("clerkId") String clerkId, @Param("enterpriseId") String enterpriseId);


    /**
     * 根据门店id查询
     *
     * @param wxEnterpriseId wx企业标识
     * @param storeId        存储id
     * @param overFlag       在国旗
     * @return {@link List }<{@link TabHaobanHmQrcode }>
     * @author mozhu
     * @date 2022-07-07 17:19:51
     */
    List<TabHaobanHmQrcode> listByStoreId(
            @Param("wxEnterpriseId") String wxEnterpriseId,
            @Param("storeId") String storeId,
            @Param("overFlag") Integer overFlag);

    /**
     * 通过id添加num
     *
     * @param hmId     hm id
     * @param overFlag 在国旗
     * @return int
     * @author mozhu
     * @date 2022-07-07 11:32:27
     */
    int updateAddNumById(@Param("hmId") Long hmId, @Param("overFlag") Integer overFlag);

    /**
     * 重置
     *
     * @author mozhu
     * @date 2022-07-07 13:54:43
     */
    void resetAddNumAndOverFlag();

    /**
     * 查询职员id 活码
     *
     * @param clerkId        职员id
     * @param wxEnterpriseId wx企业标识
     * @return {@link TabHaobanHmQrcode }
     * @author mozhu
     * @date 2022-07-07 15:55:12
     */
    TabHaobanHmQrcode queryByClerkId(@Param("clerkId") String clerkId, @Param("wxEnterpriseId") String wxEnterpriseId);

    /**
     * 查询职员id
     *
     * @param clerkId        职员id
     * @param wxEnterpriseId wx企业标识
     * @param overFlag       上限
     * @return {@link TabHaobanHmQrcode }
     * @author mozhu
     * @date 2022-07-07 17:26:34
     */
    TabHaobanHmQrcode queryByClerkIdAndOverFlag(@Param("clerkId") String clerkId,
                                     @Param("wxEnterpriseId") String wxEnterpriseId,
                                     @Param("overFlag") Integer overFlag);

    /**
     * 活码门店ids
     *
     * @param enterpriseId 企业标识
     * @return {@link List }<{@link TabHaobanHmQrcode }>
     * @author mozhu
     * @date 2022-07-08 15:10:57
     */
    List<TabHaobanHmQrcode> getHmStoreByEnterpriseId(@Param("enterpriseId") String enterpriseId);

    /**
     * 根据分组id获取活码数量
     *
     * @param enterpriseId 企业标识
     * @param groupIds     组id
     * @return {@link List }<{@link HmQrcodeGroupNumBO }>
     * @author mozhu
     * @date 2022-07-12 17:39:12
     */
    List<HmQrcodeGroupNumBO> getGroupIdNumByEnterpriseId(@Param("enterpriseId") String enterpriseId,@Param("groupIds") List<Long> groupIds);

    /**
     * 获取总数
     *
     * @param enterpriseId 企业标识
     * @return int
     * @author mozhu
     * @date 2022-07-12 15:10:09
     */
    int getTotalByEnterpriseId(@Param("enterpriseId") String enterpriseId);

    /**
     * 批量修改 活码所属分组
     * @param ids
     * @param groupId
     * @return
     */
    int batchUpdateQrCodeGroup(@Param("ids") List<Long> ids, @Param("groupId") Long groupId);
}

