package com.gic.haoban.manage.service.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.gic.commons.util.EntityUtil;
import com.gic.haoban.common.utils.DingUtils;
import com.gic.haoban.common.utils.StringUtil;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.service.dao.mapper.WxEnterpriseMapper;
import com.gic.haoban.manage.service.dao.mapper.WxEnterpriseRelatedMapper;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterprise;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterpriseRelated;
import com.gic.haoban.manage.service.service.WxEnterpriseService;

@Service
public class WxEnterpriseServiceImpl implements WxEnterpriseService {
    private static org.apache.logging.log4j.Logger logger= LogManager.getLogger(WxEnterpriseService.class);

    @Autowired
    private WxEnterpriseMapper mapper;
    @Autowired
    private WxEnterpriseRelatedMapper wxEnterpriseRelatedMapper ;

    @Override
    public String add(WxEnterpriseDTO wxDTO) {
        Date now = new Date();
        wxDTO.setWxEnterpriseId(StringUtil.randomUUID());
        wxDTO.setCreateTime(now);
        wxDTO.setUpdateTime(now);
        TabHaobanWxEnterprise tabHaobanWxEnterprise = EntityUtil.changeEntityByJSON(TabHaobanWxEnterprise.class, wxDTO);
        tabHaobanWxEnterprise.setStatusFlag(1);
        tabHaobanWxEnterprise.setBindFlag(1);
        mapper.insertSelective(tabHaobanWxEnterprise);
        return wxDTO.getWxEnterpriseId();
    }

    @Override
    public WxEnterpriseDTO selectById(String wxEnterpriseId) {
        if (StringUtils.isBlank(wxEnterpriseId)) {
            return null;
        }
        return EntityUtil.changeEntityByJSON(WxEnterpriseDTO.class, mapper.selectByPrimaryKey(wxEnterpriseId));
    }

    @Override
    public TabHaobanWxEnterprise getEnterpriseBycorpId(String corpId) {
        if (StringUtils.isBlank(corpId)) {
            return null;
        }
        return mapper.getEnterpriseBycorpId(corpId, 1);
    }

    @Override
    public TabHaobanWxEnterprise getDelEnterpriseBycorpId(String corpId) {
        return mapper.getEnterpriseBycorpId(corpId, 0);
    }

    @Override
    public void update(WxEnterpriseDTO enterpriseDTO) {
        enterpriseDTO.setUpdateTime(new Date());
        mapper.updateByPrimaryKeySelective(EntityUtil.changeEntityByJSON(TabHaobanWxEnterprise.class, enterpriseDTO));
    }

    @Override
    public int delete(String wxEnterpriseId) {
        TabHaobanWxEnterprise enterprise = new TabHaobanWxEnterprise();
        enterprise.setWxEnterpriseId(wxEnterpriseId);
        enterprise.setStatusFlag(0);
        enterprise.setUpdateTime(new Date());
        return this.mapper.updateByPrimaryKeySelective(enterprise);
    }

    @Override
    public int unbind(String wxEnterpriseId) {
        TabHaobanWxEnterprise enterprise = new TabHaobanWxEnterprise();
        enterprise.setWxEnterpriseId(wxEnterpriseId);
        enterprise.setBindFlag(0);
        enterprise.setUpdateTime(new Date());
        return this.mapper.updateByPrimaryKeySelective(enterprise);
    }

    @Override
    public TabHaobanWxEnterprise getEnterpriseBycorpIdNoStatus(String corpId) {
        return this.mapper.getEnterpriseBycorpId(corpId, null);
    }

    @Override
    public TabHaobanWxEnterprise getEnterpriseByCorpName(String corpName) {
        if (StringUtils.isBlank(corpName)) {
            return null;
        }
        return this.mapper.getEnterpriseByCorpName(corpName);
    }

    @Override
    public List<TabHaobanWxEnterprise> listAll() {
        return mapper.listAll();
    }


    @Override
    public int updateUseridFlagById(Integer useridFlag, String corpid, String wxEnterpriseId) {
        return mapper.updateUseridFlagById(useridFlag, corpid, wxEnterpriseId);
    }
    @Override
    public void stopHaoban(String enterpriseId) {
        List<TabHaobanWxEnterpriseRelated> list =  this.wxEnterpriseRelatedMapper.listByEnterpriseId(enterpriseId) ;
        logger.info("停用好办={},list={}",enterpriseId,JSON.toJSONString(list));
        if(CollectionUtils.isEmpty(list)){
            return ;
        }
        List<String> wxEnterpriseIdList = new ArrayList<>();
        for(TabHaobanWxEnterpriseRelated item : list) {
            String wxEnterpriseId = item.getWxEnterpriseId() ;
            wxEnterpriseIdList.add(wxEnterpriseId) ;
        }
        this.wxEnterpriseRelatedMapper.deleteByEnterpriseId(enterpriseId) ;
        Map<String,Object> enMap = new HashMap<>();
        enMap.put("enterpriseId", enterpriseId) ;
        alert("删除商户企微关联告警", enMap);
        for(String wxEnterpriseId : wxEnterpriseIdList) {
          List<TabHaobanWxEnterpriseRelated> relationList =  this.wxEnterpriseRelatedMapper.listByWxenterpriseId(wxEnterpriseId) ;
          if(CollectionUtils.isEmpty(relationList)) {
              // 停用企微
              TabHaobanWxEnterprise tab = new TabHaobanWxEnterprise() ;
              tab.setWxEnterpriseId(wxEnterpriseId);
              tab.setStatusFlag(0);
              tab.setUpdateTime(new Date());
              this.mapper.updateByPrimaryKeySelective(tab) ;
              Map<String,Object> map = new HashMap<>();
              map.put("wxEnterpriseId", wxEnterpriseId) ;
              alert("删除企微告警", map);
          }
        }
    }
    
    private void alert(String title, Map<String, Object> map) {
        String msg = title + JSON.toJSONString(map);
        String dingUrl = "https://oapi.dingtalk.com/robot/send?access_token=c38fdc53d26e9a019640755bdada1ce07ebd44a2555d1c8acc299de7a7b5b857";
        DingUtils.send(msg, dingUrl, true);
    }
}
