package com.gic.haoban.manage.service.service.out.impl.hm;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.errCode.HaoBanErrCodeCommon;
import com.gic.haoban.manage.api.dto.StaffClerkRelationDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.dto.hm.HmClerkRelationDTO;
import com.gic.haoban.manage.api.dto.hm.HmQrcodeDTO;
import com.gic.haoban.manage.api.dto.hm.HmQrcodeStoreDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmClerkRelationQDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmQrcodeQDTO;
import com.gic.haoban.manage.api.enums.hm.HmWelcomeReferType;
import com.gic.haoban.manage.api.service.hm.HmQrcodeApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterprise;
import com.gic.haoban.manage.service.pojo.bo.hm.HmClerkRelationBO;
import com.gic.haoban.manage.service.pojo.bo.hm.HmGroupSettingBO;
import com.gic.haoban.manage.service.pojo.bo.hm.HmQrcodeBO;
import com.gic.haoban.manage.service.pojo.bo.hm.HmWelcomeRelationBO;
import com.gic.haoban.manage.service.pojo.bo.welcome.WelcomeDetailBO;
import com.gic.haoban.manage.service.service.StaffClerkRelationService;
import com.gic.haoban.manage.service.service.StaffService;
import com.gic.haoban.manage.service.service.WelcomeService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.service.hm.HmClerkRelationService;
import com.gic.haoban.manage.service.service.hm.HmGroupService;
import com.gic.haoban.manage.service.service.hm.HmQrcodeService;
import com.gic.haoban.manage.service.service.hm.WelcomeRelationService;
import com.gic.haoban.manage.service.service.impl.log.LogRecordAfterServiceImpl;
import com.gic.haoban.manage.service.util.QwFriendLimitCountUtil;
import com.gic.log.record.anno.GicLogRecord;
import com.gic.log.record.util.GicLogRecordCategoryEnum;
import com.gic.log.record.util.GicLogRecordEvaluationContext;
import com.gic.log.record.util.GicLogRecordOptTypeEnum;
import com.gic.member.tag.api.dto.MemberTagDTO;
import com.gic.member.tag.api.service.MemberTagApiService;
import com.gic.redis.data.util.RedisUtil;
import com.gic.wechat.api.dto.qywx.response.QywxResponseDTO;
import com.gic.wechat.api.dto.qywx.welcome.QywxExternalcontactDTO;
import com.gic.wechat.api.dto.qywx.welcome.QywxExternalcontactResultDTO;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @author mozhu
 * @date 2022/7/5 09:34
 */
@Service
public class HmQrcodeApiServiceImpl implements HmQrcodeApiService {

    private static Logger logger = LoggerFactory.getLogger(HmQrcodeApiServiceImpl.class);

    @Autowired
    private HmQrcodeService hmQrcodeService;
    @Autowired
    private QywxUserApiService qywxUserApiService;
    @Autowired
    private WxEnterpriseService wxEnterpriseService;
    @Autowired
    private Config config;
    @Autowired
    private StaffClerkRelationService staffClerkRelationService;
    @Autowired
    private HmClerkRelationService hmClerkRelationService;
    @Autowired
    private ClerkService clerkService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private StaffService staffService;
    @Autowired
    private WelcomeRelationService welcomeRelationService;
    @Autowired
    private HmGroupService hmGroupService;
    @Autowired
    private MemberTagApiService memberTagApiService;
    @Autowired
    private WelcomeService welcomeService;


    @GicLogRecord(value = "新增员工活码-${#logContent}",
            category = GicLogRecordCategoryEnum.HUOMA,
            optType = GicLogRecordOptTypeEnum.HUOMA_ADD,
            optPage = "活码管理-新增员工活码",
            afterFunc = LogRecordAfterServiceImpl.class)
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse add(HmQrcodeQDTO hmQrcodeQDTO) {
        String wxEnterpriseId = hmQrcodeQDTO.getWxEnterpriseId();
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        if (wxEnterpriseDTO == null) {
            logger.error("未查询到微信企业");
            return ServiceResponse.failure(Convert.toStr(HaoBanErrCodeCommon.ERR_8.getCode()),"微信企业不存在");
        }

        hmQrcodeQDTO.setHmId(UniqueIdUtils.uniqueLong());
        Integer hmType = hmQrcodeQDTO.getHmType();
        Long hmId = hmQrcodeQDTO.getHmId();
        if (hmType == 1) {
            hmQrcodeQDTO.setHmCode("HM01" + hmId);
        } else {
            hmQrcodeQDTO.setHmCode("HM02" + hmId);
        }
        Long hmGroupId = hmQrcodeQDTO.getHmGroupId();
        if (hmGroupId == null) {
            hmQrcodeQDTO.setHmGroupId(0L);
        }
        List<String> clerkIdList = setStoreClerk(hmQrcodeQDTO, wxEnterpriseId, hmId, hmType);
        if (CollectionUtils.isEmpty(clerkIdList)) {
            return ServiceResponse.failure(Convert.toStr(HaoBanErrCodeCommon.ERR_8.getCode()),"该导购已经生成过活码或未关联过好办");
        }
        //生成活码
        QywxExternalcontactDTO qywxExternalcontactDTO = new QywxExternalcontactDTO();
        qywxExternalcontactDTO.setType(hmQrcodeQDTO.getHmType());
        qywxExternalcontactDTO.setScene(2);
        qywxExternalcontactDTO.setRemark(hmQrcodeQDTO.getName());
        qywxExternalcontactDTO.setSkipVerify(hmQrcodeQDTO.getPassFlag() == 1);
        qywxExternalcontactDTO.setState("HM" + hmId);
        List<String> userIds = staffClerkRelationService.listWxUserIdByClerkIds(clerkIdList, wxEnterpriseId);
        if (CollectionUtils.isEmpty(userIds)) {
            return ServiceResponse.failure(Convert.toStr(HaoBanErrCodeCommon.ERR_8.getCode()),"导购未关联好办");
        }
        qywxExternalcontactDTO.setUser(userIds);
        logger.info("新增活码参数：{}", JSON.toJSONString(qywxExternalcontactDTO));
        QywxExternalcontactResultDTO qywxExternalcontactResultDTO = qywxUserApiService.addContactWay(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), qywxExternalcontactDTO);
        if (qywxExternalcontactResultDTO != null && "0".equals(qywxExternalcontactResultDTO.getErrcode())) {
            hmQrcodeQDTO.setWxQrcode(qywxExternalcontactResultDTO.getQr_code());
            hmQrcodeQDTO.setWxConfigId(qywxExternalcontactResultDTO.getConfig_id());
        }
        int saveResult = hmQrcodeService.insert(hmQrcodeQDTO);
        if (saveResult == 1) {
            // save welcome relation
            this.updateWelcomeRelation(hmQrcodeQDTO);
        }
        setLoggerContext(hmQrcodeQDTO.getEnterpriseId(), wxEnterpriseId, hmQrcodeQDTO.getCreatorId(), hmQrcodeQDTO.getCreatorName());
        GicLogRecordEvaluationContext.putAttribute("logContent", "【" + hmId + "-" + hmQrcodeQDTO.getName() + "】");
        return ServiceResponse.success();
    }

    /**
     * 设置日志记录器上下文
     * 日志
     *
     * @param enterpriseId   企业标识
     * @param wxEnterpriseId wx企业标识
     * @param userId         用户id
     * @param userName       用户名
     * @author mozhu
     * @date 2022-07-12 14:20:44
     */
    private void setLoggerContext(String enterpriseId, String wxEnterpriseId, String userId, String userName) {
        GicLogRecordEvaluationContext.putAttribute("enterpriseId", enterpriseId);
        GicLogRecordEvaluationContext.putAttribute("wxEnterpriseId", wxEnterpriseId);
        GicLogRecordEvaluationContext.putAttribute("userId", userId);
        GicLogRecordEvaluationContext.putAttribute("userName", userName);
    }


    @GicLogRecord(value = "修改员工活码-${#hmName}-${#logContent}",
            category = GicLogRecordCategoryEnum.HUOMA,
            optType = GicLogRecordOptTypeEnum.HUOMA_EDIT,
            optPage = "活码管理-修改员工活码",
            afterFunc = LogRecordAfterServiceImpl.class)
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse update(HmQrcodeQDTO hmQrcodeQDTO) {
        String wxEnterpriseId = hmQrcodeQDTO.getWxEnterpriseId();
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        if (wxEnterpriseDTO == null) {
            logger.error("未查询到微信企业");
            return ServiceResponse.failure(Convert.toStr(HaoBanErrCodeCommon.ERR_8.getCode()),"微信企业不存在");
        }

        Long hmId = hmQrcodeQDTO.getHmId();
        Integer hmType = hmQrcodeQDTO.getHmType();
        List<String> clerkIdList = setStoreClerk(hmQrcodeQDTO, wxEnterpriseId, hmId, hmType);
        if (CollectionUtils.isEmpty(clerkIdList)) {
            return ServiceResponse.failure(Convert.toStr(HaoBanErrCodeCommon.ERR_8.getCode()),"该导购已经生成过活码或未关联过好办");
        }
        //更新活码
        QywxExternalcontactDTO qywxExternalcontactDTO = new QywxExternalcontactDTO();
        qywxExternalcontactDTO.setType(hmQrcodeQDTO.getHmType());
        qywxExternalcontactDTO.setScene(2);
        qywxExternalcontactDTO.setRemark(hmQrcodeQDTO.getName());
        qywxExternalcontactDTO.setSkipVerify(hmQrcodeQDTO.getPassFlag() == 1);
        qywxExternalcontactDTO.setState("HM" + hmId);
        List<String> userIds = staffClerkRelationService.listWxUserIdByClerkIds(clerkIdList, wxEnterpriseId);
        if (CollectionUtils.isEmpty(userIds)) {
            return ServiceResponse.failure(Convert.toStr(HaoBanErrCodeCommon.ERR_8.getCode()),"导购未关联好办");
        }
        qywxExternalcontactDTO.setUser(userIds);
        QywxResponseDTO qywxResponseDTO = qywxUserApiService.updateContactWay(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), qywxExternalcontactDTO);
        if (qywxResponseDTO.getErrcode() != 0) {
            logger.error("更新企微活码失败");
            return ServiceResponse.failure(Convert.toStr(HaoBanErrCodeCommon.ERR_0.getCode()),qywxResponseDTO.getErrmsg());
        }
        
        int updateResult = hmQrcodeService.update(hmQrcodeQDTO);
        if (updateResult == 1) {
            // 更新欢迎语引用次数
            this.updateWelcomeRelation(hmQrcodeQDTO);
        }

        setLoggerContext(hmQrcodeQDTO.getEnterpriseId(), wxEnterpriseId, hmQrcodeQDTO.getModifierId(), hmQrcodeQDTO.getModifierName());

        updateSystemLog(hmQrcodeQDTO, hmId);
        return ServiceResponse.success();
    }

    /**
     * 修改活码操作日志
     * @param hmQrcodeQDTO
     * @param hmId
     */
    private void updateSystemLog(HmQrcodeQDTO hmQrcodeQDTO, Long hmId) {
        StringBuffer logContent = new StringBuffer();
        HmQrcodeBO hmOld = hmQrcodeService.queryById(hmId);
        //操作日志
        //名称
        String name = hmQrcodeQDTO.getName();
        String nameOld = hmOld.getName();
        if (!name.equals(nameOld)) {
            logContent.append("活码名称「").append(nameOld).append("」").append("变更为「").append(name).append("」");
        }
        //分组
        Long hmGroupId = hmQrcodeQDTO.getHmGroupId();
        Long hmGroupIdOld = hmOld.getHmGroupId();
        if (!hmGroupId.equals(hmGroupIdOld)) {
            HmGroupSettingBO hmGroupSettingBOOld = hmGroupService.queryGroupSettingDetail(hmGroupIdOld);
            HmGroupSettingBO hmGroupSettingBO = hmGroupService.queryGroupSettingDetail(hmGroupId);
            logContent.append("活码分组「").append(hmGroupSettingBOOld.getGroupName()).append("」").append("变更为「").append(hmGroupSettingBO.getGroupName()).append("」");
        }
        //员工变更
        String staffId = hmQrcodeQDTO.getStaffId();
        String staffIdOld = hmOld.getStaffId();
        if (!staffId.equals(staffIdOld)) {
            logContent.append("新增使用成员「").append(hmQrcodeQDTO.getStaffName()).append("」").append("删除使用成员「").append(hmOld.getStaffName()).append("」");
        }
        Integer passFlag = hmQrcodeQDTO.getPassFlag();
        Integer passFlagOld = hmOld.getPassFlag();
        if (!passFlag.equals(passFlagOld)) {
            if (passFlag == 1) {
                logContent.append("关闭自动通过好友").append("变更为开启自动通过好友");
            } else {
                logContent.append("开启自动通过好友").append("变更为关闭自动通过好友");
            }
        }
        //标签
        String memberLabelId = hmQrcodeQDTO.getMemberLabelId();
        String memberLabelIdOld = hmOld.getMemberLabelId();
        if (!memberLabelId.equals(memberLabelIdOld)) {
            MemberTagDTO memberTagDTO = memberTagApiService.getMemberTagById(memberLabelId);
            MemberTagDTO memberTagDTOOld = memberTagApiService.getMemberTagById(memberLabelIdOld);
            logContent.append("活码标签「").append(memberTagDTOOld.getTagName()).append("」").append("变更为「").append(memberTagDTO.getTagName()).append("」");
        }

        //欢迎语
        String welcomeId = hmQrcodeQDTO.getWelcomeId();
        String welcomeIdOld = hmOld.getWelcomeId();
        if (StringUtils.isBlank(welcomeId) && StringUtils.isNotBlank(welcomeIdOld)) {
            logContent.append("欢迎语变更为「 不发送欢迎语」");
        }
        if (StringUtils.isNotBlank(welcomeId) && StringUtils.isBlank(welcomeIdOld)) {
            WelcomeDetailBO welcome = welcomeService.getWelcome(welcomeId, hmQrcodeQDTO.getEnterpriseId());
            logContent.append("欢迎语变更为「").append(welcome.getTitle()).append("」");
        }
        if (StringUtils.isNotBlank(welcomeId) && StringUtils.isNotBlank(welcomeIdOld)) {
            if (!welcomeId.equals(welcomeIdOld)) {
                WelcomeDetailBO welcome = welcomeService.getWelcome(welcomeId, hmQrcodeQDTO.getEnterpriseId());
                WelcomeDetailBO welcomeOld = welcomeService.getWelcome(welcomeIdOld, hmQrcodeQDTO.getEnterpriseId());
                logContent.append("欢迎语「").append(welcomeOld.getTitle()).append("」").append("变更为「").append(welcome.getTitle()).append("」");
            }
        }
        GicLogRecordEvaluationContext.putAttribute("hmName", "【" + hmId + "-" + hmQrcodeQDTO.getName() + "】：");
        GicLogRecordEvaluationContext.putAttribute("logContent", logContent.toString());
    }


    /**
     * 更新活码 欢迎语引用关系
     *
     * @param hmQrcodeQDTO
     */
    private void updateWelcomeRelation(HmQrcodeQDTO hmQrcodeQDTO) {
        HmWelcomeRelationBO welcomeRelationBo = new HmWelcomeRelationBO();
        welcomeRelationBo.setEnterpriseId(hmQrcodeQDTO.getEnterpriseId());
        welcomeRelationBo.setWxEnterpriseId(hmQrcodeQDTO.getWxEnterpriseId());
        welcomeRelationBo.setWelcomeId(hmQrcodeQDTO.getWelcomeId());
        welcomeRelationBo.setReferId(hmQrcodeQDTO.getHmId() + "");
        welcomeRelationBo.setReferCode(hmQrcodeQDTO.getHmCode());
        welcomeRelationBo.setReferName(hmQrcodeQDTO.getName());
        welcomeRelationBo.setType(HmWelcomeReferType.HM.getCode());
        welcomeRelationService.updateWelcomeRelation(welcomeRelationBo);
    }

    /**
     * 设置门店和导购
     *
     * @param hmQrcodeQDTO
     * @param wxEnterpriseId
     * @param hmId
     * @param hmType
     * @return
     */
    private List<String> setStoreClerk(HmQrcodeQDTO hmQrcodeQDTO, String wxEnterpriseId, Long hmId, Integer hmType) {
        List<String> clerkIdList = hmQrcodeQDTO.getClerkIdList();
        if (hmType == 2) {
            //多人活码
            insertHmClerkRelation(hmQrcodeQDTO, wxEnterpriseId, hmId, clerkIdList);
        } else if (hmType == 1) {
            //单人活码
            String clerkId = clerkIdList.get(0);
            HmQrcodeBO hmQrcodeBO = hmQrcodeService.queryByClerkId(clerkId, wxEnterpriseId);
            if (hmQrcodeBO != null) {
                logger.error("该导购已经生成过活码：{}", clerkId);
                return null;
            }
            hmQrcodeQDTO.setClerkId(clerkId);
            ClerkDTO clerkDTO = clerkService.getclerkById(clerkId);
            if (clerkDTO != null) {
                hmQrcodeQDTO.setClerkName(clerkDTO.getClerkName());
                hmQrcodeQDTO.setClerkCode(clerkDTO.getClerkCode());
                String storeId = clerkDTO.getStoreId();
                hmQrcodeQDTO.setStoreId(storeId);
                StoreDTO storeDTO = storeService.getStore(storeId);
                if (storeDTO != null) {
                    hmQrcodeQDTO.setStoreCode(storeDTO.getStoreCode());
                    hmQrcodeQDTO.setStoreName(storeDTO.getStoreName());
                }
            }
            StaffClerkRelationDTO staffClerkRelationDTO = staffClerkRelationService.getByClerkId(clerkId);
            if (staffClerkRelationDTO != null) {
                hmQrcodeQDTO.setStaffId(staffClerkRelationDTO.getStaffId());
                TabHaobanStaff tabHaobanStaff = staffService.selectById(staffClerkRelationDTO.getStaffId());
                if (tabHaobanStaff != null) {
                    hmQrcodeQDTO.setStaffName(tabHaobanStaff.getStaffName());
                    hmQrcodeQDTO.setWxUserId(tabHaobanStaff.getWxUserId());
                }
            }
        }
        return clerkIdList;
    }

    /**
     * 多人活码关联关系
     *
     * @param hmQrcodeQDTO
     * @param wxEnterpriseId
     * @param hmId
     * @param clerkIdList
     */
    private void insertHmClerkRelation(HmQrcodeQDTO hmQrcodeQDTO, String wxEnterpriseId, Long hmId, List<String> clerkIdList) {
        List<HmClerkRelationQDTO> clerkRelationQDTOS = new ArrayList<>();
        for (String clerkId : clerkIdList) {
            StaffClerkRelationDTO clerkRelationDTO = staffClerkRelationService.getByClerkId(clerkId);
            HmClerkRelationQDTO hmClerkRelationQDTO = new HmClerkRelationQDTO();
            hmClerkRelationQDTO.setRelationId(UniqueIdUtils.uniqueLong());
            hmClerkRelationQDTO.setHmId(hmId);
            hmClerkRelationQDTO.setWxUserId(clerkRelationDTO.getWxUserId());
            hmClerkRelationQDTO.setWxEnterpriseId(wxEnterpriseId);
            hmClerkRelationQDTO.setEnterpriseId(hmQrcodeQDTO.getEnterpriseId());
            hmClerkRelationQDTO.setClerkId(clerkId);
            ClerkDTO clerkDTO = clerkService.getclerkById(clerkId);
            if (clerkDTO != null) {
                String storeId = clerkDTO.getStoreId();
                hmClerkRelationQDTO.setStoreId(storeId);
                StoreDTO storeDTO = storeService.getStore(storeId);
                if (storeDTO != null) {
                    hmClerkRelationQDTO.setStoreCode(storeDTO.getStoreCode());
                    hmClerkRelationQDTO.setStoreName(storeDTO.getStoreName());
                }
                hmClerkRelationQDTO.setClerkName(clerkDTO.getClerkName());
                hmClerkRelationQDTO.setClerkCode(clerkDTO.getClerkCode());
            }

            StaffClerkRelationDTO staffClerkRelationDTO = staffClerkRelationService.getByClerkId(clerkId);
            if (staffClerkRelationDTO != null) {
                hmClerkRelationQDTO.setStaffId(staffClerkRelationDTO.getStaffId());
                TabHaobanStaff tabHaobanStaff = staffService.selectById(staffClerkRelationDTO.getStaffId());
                if (tabHaobanStaff != null) {
                    hmClerkRelationQDTO.setStaffName(tabHaobanStaff.getStaffName());
                }
            }
            clerkRelationQDTOS.add(hmClerkRelationQDTO);
        }
        hmClerkRelationService.insertBatch(clerkRelationQDTOS);
    }


    @Override
    public HmQrcodeDTO queryById(Long hmId) {
        return EntityUtil.changeEntityNew(HmQrcodeDTO.class, hmQrcodeService.queryById(hmId));
    }

    @GicLogRecord(value = "废弃员工活码-${#logContent}",
            category = GicLogRecordCategoryEnum.HUOMA,
            optType = GicLogRecordOptTypeEnum.HUOMA_DEL,
            optPage = "活码管理-废弃员工活码",
            afterFunc = LogRecordAfterServiceImpl.class)
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse delById(Long hmId) {
        HmQrcodeBO hmQrcodeBO = hmQrcodeService.queryById(hmId);
        if (hmQrcodeBO == null) {
            return ServiceResponse.failure(Convert.toStr(HaoBanErrCodeCommon.ERR_8.getCode()),"活码不存在");
        }
        String wxConfigId = hmQrcodeBO.getWxConfigId();
        if (StringUtils.isBlank(wxConfigId)) {
            return ServiceResponse.failure(Convert.toStr(HaoBanErrCodeCommon.ERR_8.getCode()),"活码未生成");
        }
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(hmQrcodeBO.getWxEnterpriseId());
        QywxResponseDTO qywxResponseDTO = qywxUserApiService.delContactWay(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), wxConfigId);
        int errcode = qywxResponseDTO.getErrcode();
        if (errcode != 0) {
            return ServiceResponse.failure(Convert.toStr(HaoBanErrCodeCommon.ERR_8.getCode()),qywxResponseDTO.getErrmsg());
        }
        hmQrcodeService.deleteById(hmId);
        setLoggerContext(hmQrcodeBO.getEnterpriseId(), hmQrcodeBO.getWxEnterpriseId(), hmQrcodeBO.getModifierId(), hmQrcodeBO.getModifierName());
        GicLogRecordEvaluationContext.putAttribute("logContent", "【" + hmId + "-" + hmQrcodeBO.getName() + "】");
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse delByClerkId(String clerkId, String enterpriseId) {
        // 单人活码 直接废弃，多人活码，更新
        HmQrcodeBO hmQrcodeBO = hmQrcodeService.queryByClerkId(clerkId, enterpriseId);
        if (hmQrcodeBO != null) {
            delById(hmQrcodeBO.getHmId());
        }

        // 多人活码逻辑
        List<HmClerkRelationBO> hmClerkRelationBOS = hmClerkRelationService.queryByClerkId(clerkId, enterpriseId);
        if (CollectionUtils.isEmpty(hmClerkRelationBOS)) {
            logger.info("无多人活码");
            return ServiceResponse.success();
        }
        for (HmClerkRelationBO hmClerkRelationBO : hmClerkRelationBOS) {
            Long hmId = hmClerkRelationBO.getHmId();
            List<String> clerkIds = hmClerkRelationService.getClerkIdByHmId(hmId);
            clerkIds.remove(clerkId);
            // 更新活码
            HmQrcodeQDTO hmQrcodeQDTO = new HmQrcodeQDTO();
            hmQrcodeQDTO.setHmId(hmId);
            hmQrcodeQDTO.setClerkIdList(clerkIds);
            update(hmQrcodeQDTO);
        }
        return ServiceResponse.success();
    }

    @Override
    public int updateAddNumById(Long hmId, String clerkId) {
        HmQrcodeBO hmQrcodeBO = hmQrcodeService.queryById(hmId);
        if (hmQrcodeBO == null) {
            logger.error("无活码数据");
            return 0;
        }
        Integer hmType = hmQrcodeBO.getHmType();
        int limit = QwFriendLimitCountUtil.getLimit(true);
        Integer addNum = hmQrcodeBO.getAddNum();
        if (hmType == 1) {
            //单人活码
            Integer overFlag = 0;
            if (addNum + 1 >= limit) {
                overFlag = 1;
            }
            return hmQrcodeService.updateAddNumById(hmId, overFlag);
        } else if (hmType == 2) {
            //多人活码
            HmClerkRelationBO hmClerkRelationBO = hmClerkRelationService.queryByHmIdAndClerkId(hmId, clerkId);
            if (hmClerkRelationBO == null) {
                logger.error("无多人活码导购数据,clerkId:{},hmId:{}", clerkId, hmId);
                return 0;
            }
            Integer overFlag = 0;
            if (addNum + 1 >= limit) {
                overFlag = 1;
            }
            Long relationId = hmClerkRelationBO.getRelationId();
            return hmClerkRelationService.updateAddNumById(relationId, overFlag);
        }
        return 0;
    }

    @Override
    public void resetAddNumAndOverFlag(String param) {
        logger.info("重置上限时间定时任务 start");
        hmQrcodeService.resetAddNumAndOverFlag();
        hmClerkRelationService.resetAddNumAndOverFlag();
        logger.info("重置上限时间定时任务 end");
    }

    @Override
    public void checkStaffOpenContact(String param) {
        logger.info("检查员工是否开启联系我,定时任务，start");
        List<TabHaobanWxEnterprise> tabHaobanWxEnterprises = wxEnterpriseService.listAll();
        for (TabHaobanWxEnterprise tabHaobanWxEnterpris : tabHaobanWxEnterprises) {
            //查询出所有正常的导购
            String wxEnterpriseId = tabHaobanWxEnterpris.getWxEnterpriseId();
            //企微已经开启联系我
            String wxUserIds = qywxUserApiService.listCorpExternalUser(tabHaobanWxEnterpris.getCorpid(), config.getWxSuiteid());
            String[] split = wxUserIds.split(",");
            List<String> wxUserIdsList = Arrays.asList(split);
            List<StaffClerkRelationDTO> staffClerkRelationDTOS = staffClerkRelationService.listIdsByWxOpenUserIds(wxUserIdsList, wxEnterpriseId);
            if (CollectionUtils.isEmpty(staffClerkRelationDTOS)) {
                logger.error("无导购关联数据");
                continue;
            }
            for (StaffClerkRelationDTO staffClerkRelationDTO : staffClerkRelationDTOS) {
                //具有联系我功能的导购创建活码
                staffClerkRelationService.updateOpenConcatFlagById(1, staffClerkRelationDTO.getStaffClerkRelationId());
                //创建活码
                HmQrcodeQDTO hmQrcodeQDTO = new HmQrcodeQDTO();
                hmQrcodeQDTO.setHmType(1);
                hmQrcodeQDTO.setWxEnterpriseId(wxEnterpriseId);
                hmQrcodeQDTO.setEnterpriseId(staffClerkRelationDTO.getEnterpriseId());
                hmQrcodeQDTO.setName(staffClerkRelationDTO.getStaffName());
                hmQrcodeQDTO.setPassFlag(1);
                hmQrcodeQDTO.setStoreId(staffClerkRelationDTO.getStoreId());
                hmQrcodeQDTO.setClerkIdList(Collections.singletonList(staffClerkRelationDTO.getClerkId()));
                add(hmQrcodeQDTO);
            }

            List<StaffClerkRelationDTO> relationDTOS = staffClerkRelationService.listIdsByNotInWxOpenUserIds(wxUserIdsList, wxEnterpriseId);
            if (CollectionUtils.isEmpty(relationDTOS)) {
                logger.error("无导购不包含关联数据");
                continue;
            }
            for (StaffClerkRelationDTO relationDTO : relationDTOS) {
                //有活码，但是企微联系我功能 被关闭
                String clerkId = relationDTO.getClerkId();
                HmQrcodeBO hmQrcodeBO = hmQrcodeService.queryByClerkId(clerkId, relationDTO.getWxEnterpriseId());
                if (hmQrcodeBO == null) {
                    logger.info("导购无活码，无需处理：clerkId：{}", clerkId);
                    continue;
                }
                hmQrcodeService.updateStatusById(hmQrcodeBO.getHmId(), 3);
            }
        }
        logger.info("检查员工是否开启联系我,定时任务，end");
    }

    @Override
    public List<HmQrcodeStoreDTO> getHmStoreByEnterpriseId(String enterpriseId) {
        String key = "HM:STORE:" + enterpriseId;
        Object cache = RedisUtil.getCache(key);
        if (cache != null) {
            return (List<HmQrcodeStoreDTO>) cache;
        }
        List<HmQrcodeStoreDTO> hmQrcodeStoreDTOS = EntityUtil.changeEntityListNew(HmQrcodeStoreDTO.class, hmQrcodeService.getHmStoreByEnterpriseId(enterpriseId));
        RedisUtil.setCache(key, hmQrcodeStoreDTOS, 10L, TimeUnit.MINUTES);
        return hmQrcodeStoreDTOS;
    }

    @Override
    public List<HmClerkRelationDTO> getHmClerkByHmId(Long hmId) {
        return EntityUtil.changeEntityListNew(HmClerkRelationDTO.class,hmClerkRelationService.getHmClerkByHmId(hmId));
    }

    /**
     * 批量修改活码分组
     *
     * @param hmIds
     * @param groupId
     * @return
     */
    @Override
    public ServiceResponse<Boolean> batchUpdateHmQrcodeGroup(List<Long> hmIds, Long groupId) {
        if (CollectionUtils.isEmpty(hmIds) || groupId == null){
            return ServiceResponse.failure(HaoBanErrCodeCommon.ERR_2.getCode()+"", HaoBanErrCodeCommon.ERR_2.getMsg());
        }
        int updateCount = hmQrcodeService.batchUpdateQrcodeGroup(hmIds, groupId);
        return ServiceResponse.success(updateCount > 0);
    }
}
